package dao

import (
	dbmodel "claim-monitor/model/db"
	"strings"

	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

// CreateRecord 创建领取记录
func (d *Dao) CreateRecord(record *dbmodel.Record) (err error) {
	record.Address = strings.ToLower(record.Address)
	return d.db.Clauses(clause.OnConflict{DoNothing: true}).Create(record).Error
}

// GetRecord 获取领取记录
func (d *Dao) GetRecord(address string, page, pageSize int) (records []*dbmodel.Record, total int64, err error) {
	address = strings.ToLower(address)
	err = d.db.Model(&dbmodel.Record{}).Where("address = ?", address).Count(&total).Error
	if err != nil {
		return
	}

	if total == 0 {
		return
	}

	err = d.db.Model(&dbmodel.Record{}).
		Where("address = ?", address).
		Order("`id` desc").
		Offset((page - 1) * pageSize).
		Limit(pageSize).
		Find(&records).Error

	return
}

func (d *Dao) GetClaimedAmount(address string) (amt *decimal.Decimal, err error) {
	address = strings.ToLower(address)
	amt = new(decimal.Decimal)

	err = d.db.Model(&dbmodel.Record{}).
		Where("address = ?", address).
		Select("sum(claimed_amount) as claimed_amount").
		Scan(amt).Error
	if err != nil {
		if strings.Contains(err.Error(), "could not convert value '<nil>'") {
			err = nil
		}
		return
	}

	return
}

// GetStorageHeight 获取上次缓存的高度
func (d *Dao) GetStorageHeight(key string) (value int, err error) {
	storage := new(dbmodel.Height)
	err = d.db.Model(storage).Where("`key` = ?", key).First(storage).Error
	if err == gorm.ErrRecordNotFound {
		return 1, nil
	}
	return storage.IntValue, err
}

// SetStorageHeight 设置上次缓存的高度
func (d *Dao) SetStorageHeight(key string, intValue int) (err error) {
	ret := d.db.Model(&dbmodel.Height{}).Where("`key` = ?", key).Update("int_value", intValue)
	if ret.Error != nil {
		return
	}
	if ret.RowsAffected == 0 {
		err = d.db.Create(&dbmodel.Height{
			Key:      key,
			IntValue: intValue,
		}).Error
	}
	return
}
