package dao

import (
	"context"
	"math/big"
	"time"

	"github.com/ethereum/go-ethereum"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/core/types"
)

func (d *Dao) GetBlockHeight(behindBlock ...int) (height int, err error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	defer cancel()
	n, err := d.ethClient.BlockNumber(ctx)
	if len(behindBlock) > 0 {
		n -= uint64(behindBlock[0])
		if n < 0 {
			n = 0
		}
	}
	return int(n), err
}

func (d *Dao) GetLatestBockHash() (hash string, err error) {
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer cancel()
	block, err := d.ethClient.BlockByNumber(ctx, nil)
	if err != nil {
		return
	}
	return block.Hash().Hex(), nil
}

func (d *Dao) GetBlockTime(height int) (timestamp int, err error) {
	for i := 0; i < 2; i++ {
		ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
		defer cancel()
		block, err := d.ethClient.BlockByNumber(ctx, big.NewInt(int64(height)))
		if err == nil {
			return int(block.Time()), nil
		}
	}
	return
}

func (d *Dao) GetLogs(beginHeight, endHeight int, topics []string) (logs []types.Log, err error) {
	for i := 0; i < 2; i++ {
		// 重试2次
		logs, err = d.getLogs(beginHeight, endHeight, topics)
		if err == nil {
			return logs, nil
		}
	}
	return
}

func (d *Dao) getLogs(beginHeight, endHeight int, topics []string) (logs []types.Log, err error) {
	q := ethereum.FilterQuery{
		FromBlock: big.NewInt(int64(beginHeight)),
		ToBlock:   big.NewInt(int64(endHeight)),
		Topics:    [][]common.Hash{{}},
		Addresses: []common.Address{common.HexToAddress(d.c.Chain.DistributionAddress)},
	}
	for _, topic := range topics {
		q.Topics[0] = append(q.Topics[0], common.HexToHash(topic))
	}

	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()
	return d.ethClient.FilterLogs(ctx, q)
}
