package service

import (
	"claim-monitor/model/http"
	"fmt"
	"math/big"
	"time"

	log "github.com/sirupsen/logrus"
)

func (srv *Service) GetRecord(address string, page, pageSize int) (resp *http.GetRecordResponse, err error) {
	resp = new(http.GetRecordResponse)
	records, totalCount, err := srv.d.GetRecord(address, page, pageSize)
	if err != nil {
		log.WithField("address", address).WithError(err).Error("failed to get contract")
		return
	}
	resp.Records = make([]http.Record, 0)
	ether := big.NewInt(1000000000000000000)
	for _, record := range records {
		amt, _ := big.NewFloat(0).Quo(
			big.NewFloat(0).SetInt(record.ClaimedAmount.BigInt()),
			big.NewFloat(0).SetInt(ether),
		).Float64()
		resp.Records = append(resp.Records, http.Record{
			TxHash:    record.TxHash,
			Date:      time.Unix(record.Timestamp, 0).UTC().Format("2006-01-02 15:04:05"),
			Timestamp: int(record.Timestamp),
			Amount:    fmt.Sprintf("%.6f", amt),
		})
	}

	resp.TotalCount = int(totalCount)
	return
}

func (srv *Service) GetClaimed(address string) (resp *http.GetClaimedResponse, err error) {
	resp = new(http.GetClaimedResponse)

	claimed, err := srv.d.GetClaimedAmount(address)
	if err != nil {
		log.WithField("address", address).WithError(err).Error("failed to get claimed")
		return
	}

	ether := big.NewInt(1000000000000000000)
	amt, _ := big.NewFloat(0).Quo(
		big.NewFloat(0).SetInt(claimed.BigInt()),
		big.NewFloat(0).SetInt(ether),
	).Float64()

	resp.Claimed = fmt.Sprintf("%.6f", amt)
	return
}
