package sync

import (
	dbmodel "claim-monitor/model/db"
	"strings"
	"time"

	"github.com/ethereum/go-ethereum/core/types"
	"github.com/shopspring/decimal"
	log "github.com/sirupsen/logrus"
	"gorm.io/gorm"
)

func (s *Sync) SyncClaimed(beginHeight, endHeight int) {
	if endHeight < 0 {
		return
	}

	if beginHeight < 0 {
		beginHeight = 0
	}
	claimTopics := []string{ClaimedTopic}
	logs, err := s.d.GetLogs(beginHeight, endHeight, claimTopics)
	if err != nil {
		log.WithFields(log.Fields{"begin": beginHeight, "end": endHeight}).WithError(err).Error("rpc: get logs")
		return
	}

	for _, txLog := range logs {
		s.FilterClaimed(txLog)
	}
}

func (s *Sync) FilterClaimed(txLog types.Log) {
	if len(txLog.Topics) == 0 {
		return
	}

	claimedLog, err := s.ca.ParseClaimed(txLog)
	if err != nil {
		log.WithError(err).Error("parse claimed log")
		return
	}
	claimedAmount := decimal.NewFromBigInt(claimedLog.Amount, 0)
	totalAmount := decimal.NewFromBigInt(claimedLog.TotalClaimedAmount, 0)

	timestamp, err := s.d.GetBlockTime(int(txLog.BlockNumber))
	if err != nil {
		log.WithError(err).Error("get block time")
		return
	}

	r := &dbmodel.Record{
		TxHash:        txLog.TxHash.Hex(),
		Address:       strings.ToLower(txLog.Address.String()),
		ClaimedAmount: claimedAmount,
		TotalAmount:   totalAmount,
		Timestamp:     int64(timestamp),
		CreatedAt:     time.Time{},
		UpdatedAt:     time.Time{},
		DeletedAt:     gorm.DeletedAt{},
	}

	err = s.d.CreateRecord(r)
	if err != nil {
		log.WithError(err).Error("create record")
		return
	}

	log.WithFields(log.Fields{
		"tx":      r.TxHash,
		"address": r.Address,
		"amount":  r.ClaimedAmount,
	}).Info("sync claimed record")

}
