package sync

import (
	"claim-monitor/config"
	"claim-monitor/contract"
	"claim-monitor/dao"
	"flag"
	"time"

	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/crypto"
	log "github.com/sirupsen/logrus"
)

var (
	ClaimedTopic = crypto.Keccak256Hash([]byte("Claimed(address, uint256,uint256)")).Hex()

	LastBlockKey = "last_block"
	BehindBlock  = 1
)

type Sync struct {
	c  *config.Config
	d  *dao.Dao
	ca *contract.Distribution
}

var manualSync = flag.Int("sync", 0, "sync block height")

func New(_c *config.Config, _d *dao.Dao) (sync *Sync) {
	sync = &Sync{
		c: _c,
		d: _d,
	}

	ca, err := contract.NewDistribution(common.HexToAddress(sync.c.Chain.DistributionAddress), nil)
	if err != nil {
		panic(err)
	}
	sync.ca = ca

	return sync
}

func (s *Sync) Start() {
	lastHeight, err := s.d.GetStorageHeight(LastBlockKey)
	if err != nil {
		log.WithError(err).Error("get last block height")
		return
	}

	if lastHeight != 1 {
		// 数据库里保存的是已完成的区块, 再次同步时+1
		lastHeight++
	}

	if *manualSync > 0 {
		lastHeight = *manualSync
	}

	log.WithField("height", lastHeight).Info("last sync block height")

	var latestHeight int
	var beginHeight = lastHeight
	var endHeight = beginHeight + s.c.BatchSize

	for {
		latestHeight, err = s.d.GetBlockHeight(BehindBlock)
		if err != nil {
			log.WithError(err).Error("get latest block height")
			return
		}
		if (latestHeight-s.c.BatchSize)-beginHeight < s.c.BatchSize+1 {
			time.Sleep(10 * time.Second)
			continue
		}
		s.SyncClaimed(beginHeight-s.c.BatchSize, endHeight-s.c.BatchSize)

		if err = s.d.SetStorageHeight(LastBlockKey, endHeight); err != nil {
			log.WithError(err).Error("set last block height")
		}

		beginHeight = endHeight + 1
		endHeight = beginHeight + s.c.BatchSize
		log.WithFields(log.Fields{
			"begin height":  beginHeight,
			"end height":    endHeight,
			"latest height": latestHeight,
		}).Info("sync block height")
	}
}
